/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package aokell.aspect;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.Type;

import aokell.lib.control.component.ComponentItf;
import aokell.lib.membrane.BaseType;


/**
 * This aspect introduces the features provided by the Component interface.
 * These features are introduced for all classes that implement the BaseType
 * marker interface.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public aspect AComponent {
    
    /** The reference towards the component controller. */
    private ComponentItf BaseType._fcComp;
    
    
    // -----------------------------------------------------------
    // Introduction of the Component interface
    // -----------------------------------------------------------
    
    public Type BaseType.getFcType() {
        return _fcComp.getFcType();
    }

    public Object BaseType.getFcInterface(String interfaceName)
    throws NoSuchInterfaceException {
        return _fcComp.getFcInterface(interfaceName);
    }
    
    public Object[] BaseType.getFcInterfaces() {
        return _fcComp.getFcInterfaces();
    }

    
    // ------------------------------------------------------------------
    // Introduction of the BaseType interface
    // ------------------------------------------------------------------

    public void BaseType.setFcComponent( ComponentItf _fcComp ) {
        this._fcComp = _fcComp;
    }
    
    public ComponentItf BaseType.getFcComponent() {
        return _fcComp;
    }
}
