/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package aokell.aspect;

import org.objectweb.fractal.api.Component;

import aokell.lib.control.superc.SuperControllerItf;
import aokell.lib.membrane.marker.SuperType;
import aokell.lib.util.FractalHelper;


/**
 * This aspect introduces the features provided by the super controller.
 * These features are introduced for all classes that implement the
 * SuperType marker interface.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public aspect ASuperController {
        
    // -----------------------------------------------------------
    // Introduction of the SuperController interface
    // -----------------------------------------------------------
    
    public Component[] SuperType.getFcSuperComponents() {
        SuperControllerItf _sc = FractalHelper.getSuperController(getFcComponent());
        return _sc.getFcSuperComponents();
    }
    
    // -----------------------------------------------------------
    // Introduction of the SuperControllerItf interface
    // -----------------------------------------------------------

    public void SuperType.addFcSuperComponent( Component c ) {
        SuperControllerItf _sc = FractalHelper.getSuperController(getFcComponent());
        _sc.addFcSuperComponent(c);
    }
    
    public void SuperType.removeFcSuperComponent( Component c ) {
        SuperControllerItf _sc = FractalHelper.getSuperController(getFcComponent());
        _sc.removeFcSuperComponent(c);
    }
    
}
