/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 * Contributors: Romain Rouvoy
 * -----------------------------------------------------------------------------
 * $Id: NonCompositeLifeCycleControllerImpl.java,v 1.2 2006/01/09 19:29:17 seinturier Exp $
 * -----------------------------------------------------------------------------
 */

package aokell.lib.control.lifecycle;

import java.lang.reflect.Field;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;

import aokell.lib.control.lifecycle.AbstractLifeCycleControllerImpl;
import aokell.lib.control.lifecycle.LifeCycleControllerItf;
import aokell.lib.membrane.marker.LifeCycleType;
import aokell.lib.util.FeatureHelper;
import aokell.lib.util.FractalHelper;


/**
 * Default implementation of the life cyle controller for non composite
 * components.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 */
public class NonCompositeLifeCycleControllerImpl
    extends AbstractLifeCycleControllerImpl
    implements LifeCycleControllerItf {  // Needed for fractalrmi

    // --------------------------------------------------------------
    // LifeCycleController interface methods unimplemented in
    // AbstractLifeCycleControllerImpl
    // --------------------------------------------------------------

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    public boolean setFcStarted() throws IllegalLifeCycleException {        
        if (!super.setFcStarted())
            return false;
        setFcState(true);
        return true;
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
     */
    public boolean setFcStopped() throws IllegalLifeCycleException {
        if (!super.setFcStopped())
            return false;
        setFcState(false);
        return true;
    }
    
    /**
     * @param start
     * @throws IllegalLifeCycleException
     */
    private void setFcState(final boolean start) throws IllegalLifeCycleException {
        
        Component compctrl = FeatureHelper.getFcImplementation(this.compctrl);
        
        /*
         * For Julia-like components, notify the content. AspectJ-like
         * components can not be notified: the control interface is directly
         * introduced into the content.
         */
        Object contentPart = FractalHelper.getContent(compctrl);
        if( !(contentPart instanceof LifeCycleType) &&
            contentPart instanceof LifeCycleController ) {
            if (start)
                ((LifeCycleController)contentPart).startFc();
            else
                ((LifeCycleController)contentPart).stopFc();
        }
        
        /*
         * Optimization: set the value of the introduced _isFcStarted field.
         * This field caches the lifecycle controller state.
         */
        setIsFcStartedField(start);
    }

    
    // --------------------------------------------------------------
    // Implementation specific
    // --------------------------------------------------------------
    
    private void setIsFcStartedField( boolean value )
    throws IllegalLifeCycleException {
        
        Component compctrl = FeatureHelper.getFcImplementation(this.compctrl);
        
        Object content = null;
        try {
            content = compctrl.getFcInterface("/content");
        }
        catch( NoSuchInterfaceException nsie ) {
            throw new RuntimeException(nsie);
        }
        
        Class contentClass = content.getClass();
        
        /*
         * Get the _isFcStarted field introduced by AspectJ.
         */
        Field f = null;
        try {
            f = contentClass.getDeclaredField(
                    "ajc$interField$aokell_aspect_AAbstractLifeCycleController$aokell_lib_membrane_marker_LifeCycleInterceptorType$_isFcStarted");
        }
        catch( NoSuchFieldException nsfe) {
            /*
             * The field does not exist for content classes which are not
             * aspectized by AspectJ. This is the case for Julia legacy code.
             */
            return;
        }
        
        /*
         * Set the value of the _isFcStarted field.
         */
        try {
            f.setBoolean(content,value);
        }
        catch( IllegalAccessException iae ) {
            throw new IllegalLifeCycleException(iae.getMessage());
        }
    }
    
}
