/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package aokell.lib.interf;

import org.objectweb.fractal.api.Interface;

import aokell.lib.asm.ASMGeneratorForBoundableMInterface;



/**
 * This class generates {@link Interface} implementations for control components
 * which are bound to a target interface. This is typically the case of
 * interfaces associated to composite components, or of client interfaces
 * associated to primitive components.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public class DelegatorForBoundableMInterface extends Delegator {

    /** The singleton instance of itself. */
    private static DelegatorForBoundableMInterface singleton =
        new DelegatorForBoundableMInterface();
    
    private DelegatorForBoundableMInterface() {}
    
    public static DelegatorForBoundableMInterface get() {
        return singleton;
    }

    
    /**
     * Generate a proxy that implements the same interface as a delegate and
     * that extends the InterfaceImpl class.
     */
    protected ClassDefinition generateDelegator(
            String targetClassname, String delegateClassname ) {
        
        return generator.generateDelegator(targetClassname,delegateClassname);
    }
    
    private ASMGeneratorForBoundableMInterface generator =
        new ASMGeneratorForBoundableMInterface();

}
