/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package aokell;
import org.objectweb.fractal.api.control.NameController;

import aokell.lib.control.attribute.AttributeControllerImpl;
import aokell.lib.control.attribute.AttributeControllerItf;
import aokell.lib.control.binding.BindingControllerItf;
import aokell.lib.control.binding.CompositeAutoBindingControllerImpl;
import aokell.lib.control.binding.CompositeBindingControllerImpl;
import aokell.lib.control.binding.CompositeTemplateBindingControllerImpl;
import aokell.lib.control.binding.FlatBindingControllerImpl;
import aokell.lib.control.binding.FlatTemplateBindingControllerImpl;
import aokell.lib.control.binding.PrimitiveAutoBindingControllerImpl;
import aokell.lib.control.binding.PrimitiveBindingControllerImpl;
import aokell.lib.control.binding.PrimitiveTemplateBindingControllerImpl;
import aokell.lib.control.component.ComponentImpl;
import aokell.lib.control.component.ComponentItf;
import aokell.lib.control.component.CompositeComponentImpl;
import aokell.lib.control.content.ContentControllerImpl;
import aokell.lib.control.content.ContentControllerItf;
import aokell.lib.control.content.TemplateContentControllerImpl;
import aokell.lib.control.factory.CompositeFactoryImpl;
import aokell.lib.control.factory.FactoryItf;
import aokell.lib.control.factory.PrimitiveFactoryImpl;
import aokell.lib.control.lifecycle.CompositeLifeCycleControllerImpl;
import aokell.lib.control.lifecycle.LifeCycleControllerItf;
import aokell.lib.control.lifecycle.NonCompositeLifeCycleControllerImpl;
import aokell.lib.control.name.NameControllerImpl;
import aokell.lib.control.name.NameControllerItf;
import aokell.lib.control.superc.SuperControllerImpl;
import aokell.lib.control.superc.SuperControllerItf;
import aokell.lib.membrane.BaseType;
import aokell.lib.membrane.ControllerDef;
import aokell.lib.membrane.MembraneDef;
import aokell.lib.membrane.composite.CompositeTemplateType;
import aokell.lib.membrane.composite.CompositeType;
import aokell.lib.membrane.composite.ParametricCompositeTemplateType;
import aokell.lib.membrane.composite.ParametricCompositeType;
import aokell.lib.membrane.flat.FlatTemplateType;
import aokell.lib.membrane.flat.FlatType;
import aokell.lib.membrane.flat.ParametricFlatTemplateType;
import aokell.lib.membrane.flat.ParametricFlatType;
import aokell.lib.membrane.marker.LifeCycleInterceptorType;
import aokell.lib.membrane.primitive.ParametricPrimitiveTemplateType;
import aokell.lib.membrane.primitive.ParametricPrimitiveType;
import aokell.lib.membrane.primitive.PrimitiveTemplateType;
import aokell.lib.membrane.primitive.PrimitiveType;


/**
 * The class contains a single field which defines the membranes supported by
 * AOKell.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public class AOKellMembranes {

    /** AOKell membranes. */
    final public static MembraneDef[] MEMBRANES =
        new MembraneDef[]{
            new MembraneDef(
                    "bootstrap",
                    new ControllerDef[] {
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    ComponentImpl.class
                            )
                    },
                    BaseType.class,
                    new Class[]{}),
            
            new MembraneDef(
                    "flatPrimitive",
                    new ControllerDef[] {
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    FlatBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    ComponentImpl.class
                            ),
                            new ControllerDef(
                                    LifeCycleControllerItf.NAME,
                                    LifeCycleControllerItf.TYPE,
                                    LifeCycleControllerItf.class,
                                    NonCompositeLifeCycleControllerImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            )
                    },
                    FlatType.class,
                    new Class[]{LifeCycleInterceptorType.class}),
            new MembraneDef(
                    "primitive",
                    new ControllerDef[] {
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    PrimitiveBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    ComponentImpl.class
                            ),
                            new ControllerDef(
                                    LifeCycleControllerItf.NAME,
                                    LifeCycleControllerItf.TYPE,
                                    LifeCycleControllerItf.class,
                                    NonCompositeLifeCycleControllerImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            ),
                            new ControllerDef(
                                    SuperControllerItf.NAME,
                                    SuperControllerItf.TYPE,
                                    SuperControllerItf.class,
                                    SuperControllerImpl.class
                            )      
                    },
                    PrimitiveType.class,
                    new Class[]{LifeCycleInterceptorType.class}),
            new MembraneDef(
                    "composite",
                    new ControllerDef[] {
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    CompositeBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    CompositeComponentImpl.class
                            ),
                            new ControllerDef(
                                    ContentControllerItf.NAME,
                                    ContentControllerItf.TYPE,
                                    ContentControllerItf.class,
                                    ContentControllerImpl.class
                            ),
                            new ControllerDef(
                                    LifeCycleControllerItf.NAME,
                                    LifeCycleControllerItf.TYPE,
                                    LifeCycleControllerItf.class,
                                    CompositeLifeCycleControllerImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            ),
                            new ControllerDef(
                                    SuperControllerItf.NAME,
                                    SuperControllerItf.TYPE,
                                    SuperControllerItf.class,
                                    SuperControllerImpl.class
                            )      
                    },
                    CompositeType.class,
                    new Class[]{LifeCycleInterceptorType.class}),
    
            // ----------
            // Parametric
            // ----------
            new MembraneDef(
                    "flatParametricPrimitive",
                    new ControllerDef[] {
                            new ControllerDef(
                                    AttributeControllerItf.NAME,
                                    AttributeControllerItf.TYPE,
                                    AttributeControllerItf.class,
                                    AttributeControllerImpl.class
                            ),
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    FlatBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    ComponentImpl.class
                            ),
                            new ControllerDef(
                                    LifeCycleControllerItf.NAME,
                                    LifeCycleControllerItf.TYPE,
                                    LifeCycleControllerItf.class,
                                    NonCompositeLifeCycleControllerImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            )
                    },
                    ParametricFlatType.class,
                    new Class[]{LifeCycleInterceptorType.class}),
            new MembraneDef(
                    "parametricPrimitive",
                    new ControllerDef[] {
                            new ControllerDef(
                                    AttributeControllerItf.NAME,
                                    AttributeControllerItf.TYPE,
                                    AttributeControllerItf.class,
                                    AttributeControllerImpl.class
                            ),
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    PrimitiveBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    ComponentImpl.class
                            ),
                            new ControllerDef(
                                    LifeCycleControllerItf.NAME,
                                    LifeCycleControllerItf.TYPE,
                                    LifeCycleControllerItf.class,
                                    NonCompositeLifeCycleControllerImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            ),
                            new ControllerDef(
                                    SuperControllerItf.NAME,
                                    SuperControllerItf.TYPE,
                                    SuperControllerItf.class,
                                    SuperControllerImpl.class
                            )      
                    },
                    ParametricPrimitiveType.class,
                    new Class[]{LifeCycleInterceptorType.class}),
            new MembraneDef(
                    "parametricComposite",
                    new ControllerDef[] {
                            new ControllerDef(
                                    AttributeControllerItf.NAME,
                                    AttributeControllerItf.TYPE,
                                    AttributeControllerItf.class,
                                    AttributeControllerImpl.class
                            ),
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    CompositeBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    CompositeComponentImpl.class
                            ),
                            new ControllerDef(
                                    ContentControllerItf.NAME,
                                    ContentControllerItf.TYPE,
                                    ContentControllerItf.class,
                                    ContentControllerImpl.class
                            ),
                            new ControllerDef(
                                    LifeCycleControllerItf.NAME,
                                    LifeCycleControllerItf.TYPE,
                                    LifeCycleControllerItf.class,
                                    CompositeLifeCycleControllerImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            ),
                            new ControllerDef(
                                    SuperControllerItf.NAME,
                                    SuperControllerItf.TYPE,
                                    SuperControllerItf.class,
                                    SuperControllerImpl.class
                            )      
                    },
                    ParametricCompositeType.class,
                    new Class[]{LifeCycleInterceptorType.class}),
    
            // --------
            // Template
            // --------
            new MembraneDef(
                    "flatPrimitiveTemplate",
                    new ControllerDef[] {
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    FlatTemplateBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    ComponentImpl.class
                            ),
                            new ControllerDef(
                                    FactoryItf.NAME,
                                    FactoryItf.TYPE,
                                    FactoryItf.class,
                                    PrimitiveFactoryImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            )
                    },
                    FlatTemplateType.class,
                    new Class[]{}),
            new MembraneDef(
                    "primitiveTemplate",
                    new ControllerDef[] {
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    PrimitiveTemplateBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    ComponentImpl.class
                            ),
                            new ControllerDef(
                                    FactoryItf.NAME,
                                    FactoryItf.TYPE,
                                    FactoryItf.class,
                                    PrimitiveFactoryImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            ),
                            new ControllerDef(
                                    SuperControllerItf.NAME,
                                    SuperControllerItf.TYPE,
                                    SuperControllerItf.class,
                                    SuperControllerImpl.class
                            )      
                    },
                    PrimitiveTemplateType.class,
                    new Class[]{}),
            new MembraneDef(
                    "compositeTemplate",
                    new ControllerDef[] {
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    CompositeTemplateBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    CompositeComponentImpl.class
                            ),
                            new ControllerDef(
                                    ContentControllerItf.NAME,
                                    ContentControllerItf.TYPE,
                                    ContentControllerItf.class,
                                    TemplateContentControllerImpl.class
                            ),
                            new ControllerDef(
                                    FactoryItf.NAME,
                                    FactoryItf.TYPE,
                                    FactoryItf.class,
                                    CompositeFactoryImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            ),
                            new ControllerDef(
                                    SuperControllerItf.NAME,
                                    SuperControllerItf.TYPE,
                                    SuperControllerItf.class,
                                    SuperControllerImpl.class
                            )      
                    },
                    CompositeTemplateType.class,
                    new Class[]{}),
    
            // -------------------
            // Parametric template
            // -------------------
            new MembraneDef(
                    "flatParametricPrimitiveTemplate",
                    new ControllerDef[] {
                            new ControllerDef(
                                    AttributeControllerItf.NAME,
                                    AttributeControllerItf.TYPE,
                                    AttributeControllerItf.class,
                                    AttributeControllerImpl.class
                            ),
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    FlatTemplateBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    ComponentImpl.class
                            ),
                            new ControllerDef(
                                    FactoryItf.NAME,
                                    FactoryItf.TYPE,
                                    FactoryItf.class,
                                    PrimitiveFactoryImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            )
                    },
                    ParametricFlatTemplateType.class,
                    new Class[]{}),
            new MembraneDef(
                    "parametricPrimitiveTemplate",
                    new ControllerDef[] {
                            new ControllerDef(
                                    AttributeControllerItf.NAME,
                                    AttributeControllerItf.TYPE,
                                    AttributeControllerItf.class,
                                    AttributeControllerImpl.class
                            ),
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    PrimitiveTemplateBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    ComponentImpl.class
                            ),
                            new ControllerDef(
                                    FactoryItf.NAME,
                                    FactoryItf.TYPE,
                                    FactoryItf.class,
                                    PrimitiveFactoryImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            ),
                            new ControllerDef(
                                    SuperControllerItf.NAME,
                                    SuperControllerItf.TYPE,
                                    SuperControllerItf.class,
                                    SuperControllerImpl.class
                            )      
                    },
                    ParametricPrimitiveTemplateType.class,
                    new Class[]{}),
            new MembraneDef(
                    "parametricCompositeTemplate",
                    new ControllerDef[] {
                            new ControllerDef(
                                    AttributeControllerItf.NAME,
                                    AttributeControllerItf.TYPE,
                                    AttributeControllerItf.class,
                                    AttributeControllerImpl.class
                            ),
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    CompositeTemplateBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    CompositeComponentImpl.class
                            ),
                            new ControllerDef(
                                    ContentControllerItf.NAME,
                                    ContentControllerItf.TYPE,
                                    ContentControllerItf.class,
                                    TemplateContentControllerImpl.class
                            ),
                            new ControllerDef(
                                    FactoryItf.NAME,
                                    FactoryItf.TYPE,
                                    FactoryItf.class,
                                    CompositeFactoryImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            ),
                            new ControllerDef(
                                    SuperControllerItf.NAME,
                                    SuperControllerItf.TYPE,
                                    SuperControllerItf.class,
                                    SuperControllerImpl.class
                            )      
                    },
                    ParametricCompositeTemplateType.class,
                    new Class[]{}),
            
            // ------------
            // Auto-binding
            // ------------
            new MembraneDef(
                    "autoBindingPrimitive",
                    new ControllerDef[] {
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    PrimitiveAutoBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    ComponentImpl.class
                            ),
                            new ControllerDef(
                                    LifeCycleControllerItf.NAME,
                                    LifeCycleControllerItf.TYPE,
                                    LifeCycleControllerItf.class,
                                    NonCompositeLifeCycleControllerImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            ),
                            new ControllerDef(
                                    SuperControllerItf.NAME,
                                    SuperControllerItf.TYPE,
                                    SuperControllerItf.class,
                                    SuperControllerImpl.class
                            )      
                    },
                    PrimitiveType.class,
                    new Class[]{LifeCycleInterceptorType.class}),
            new MembraneDef(
                    "autoBindingComposite",
                    new ControllerDef[] {
                            new ControllerDef(
                                    BindingControllerItf.NAME,
                                    BindingControllerItf.TYPE,
                                    BindingControllerItf.class,
                                    CompositeAutoBindingControllerImpl.class
                            ),
                            new ControllerDef(
                                    ComponentItf.NAME,
                                    ComponentItf.TYPE,
                                    ComponentItf.class,
                                    CompositeComponentImpl.class
                            ),
                            new ControllerDef(
                                    ContentControllerItf.NAME,
                                    ContentControllerItf.TYPE,
                                    ContentControllerItf.class,
                                    ContentControllerImpl.class
                            ),
                            new ControllerDef(
                                    LifeCycleControllerItf.NAME,
                                    LifeCycleControllerItf.TYPE,
                                    LifeCycleControllerItf.class,
                                    CompositeLifeCycleControllerImpl.class
                            ),
                            new ControllerDef(
                                    NameControllerItf.NAME,
                                    NameControllerItf.TYPE,
                                    NameController.class,
                                    NameControllerImpl.class
                            ),
                            new ControllerDef(
                                    SuperControllerItf.NAME,
                                    SuperControllerItf.TYPE,
                                    SuperControllerItf.class,
                                    SuperControllerImpl.class
                            )      
                    },
                    CompositeType.class,
                    new Class[]{LifeCycleInterceptorType.class})
        };
    
}
