/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package aokell.lib.control.binding;

import java.util.List;

import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.type.InterfaceType;

import aokell.lib.type.InterfaceTypeImpl;

/**
 * This interface extends the Fractal BindingController interface with methods
 * to manage reverse bindings, i.e. bindings from the destination to the source,
 * and with a method to retrieve the list of currently bound collection
 * interface names.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public interface BindingControllerItf extends BindingController {

    final public static String NAME = "binding-controller";
    
    final public static InterfaceType TYPE =
        new InterfaceTypeImpl(
                NAME,
                BindingControllerItf.class.getName(),
                false, false, false);

    /**
     * Record a reverse binding. When a source interface is bound to a target
     * one, the reverse link (target -&gt; source) is also recorded in the
     * target binding controller. This reverse mappings is primarily used to
     * prevent content controllers from performing remove operations that would
     * create invalid bindings.
     * 
     * @param dstItfName  the target interface name for the binding
     * @param src         the source interface for the binding
     */
    public void reverseBindFc(String dstItfName, Object src);

    /**
     * Lookup a reverse binding. See reverseBindFc(String,Object) for the
     * rationale behind reverse bindings.
     * 
     * @param dstItfName  the target interface name to be looked up
     * @return            the corresponding source interface or null
     */
    public Object reverseLookupFc(String dstItfName);

    /**
     * Remove a reverse binding. See reverseBindFc(String,Object) for the
     * rationale behind reverse bindings.
     * 
     * @param dstItfName  the target interface name to be removed
     */
    public void reverseUnbindFc( String dstItfName );
    
    /**
     * @return  true if the given component contains at least one binding
     * (excluding the binding with the component interface).
     */
    public boolean isFcBound();
    
    /**
     * @return  true if the given component contains at least one reverse
     * binding
     */
    public boolean isFcReverseBound();        

    /**
     * Add the names of currently bound interface names to the given list.
     */
    public void addFcCollectionInterfaceNames( List cltItfNames );
}