/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package aokell.lib.control.binding;

import java.util.ArrayList;
import java.util.List;

import org.objectweb.fractal.api.NoSuchInterfaceException;

import aokell.lib.control.content.ContentControllerItf;
import aokell.lib.util.FractalHelper;

/**
 * Implementation of the binding controller for composite components.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public class CompositeBindingControllerImpl
    extends PrimitiveBindingControllerImpl
    implements BindingControllerItf {  // Needed for fractalrmi

    public CompositeBindingControllerImpl() {
        super();
    }
    
    
    // --------------------------------------------------------------
    // Implementation specific
    // --------------------------------------------------------------
    
    /**
     * Return the interface whose name is given.
     */
    protected Object getFcClientItf(String itfName)
        throws NoSuchInterfaceException {
        
        try {
            return super.getFcClientItf(itfName);
        }
        catch( NoSuchInterfaceException nsie ) {}
            
        /*
         * The interface has not been found in the external interfaces.
         * Lookup in the internal interfaces.
         * The call does not necessarily succeed and may throw NoSuchInterface
         */
        ContentControllerItf cc =
            (ContentControllerItf) FractalHelper.getContentController(compctrl);
        return cc.getFcInternalClientInterface(itfName);
    }
    
    /**
     * Return the list of client interfaces for the component attached to this
     * controller.
     */
    protected List getFcSingletonClientItfs() {
        
        List clientItfs = new ArrayList();
        
        // Add client external interfaces
        clientItfs.addAll(super.getFcSingletonClientItfs());
        
        // Add client internal interfaces (if relevant)
        ContentControllerItf cc =
            (ContentControllerItf) FractalHelper.getContentController(compctrl);
        List isbi = cc.getFcInternalSingletonClientInterfaces();
        clientItfs.addAll(isbi);
        
        return clientItfs;
    }
    
}
