/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package aokell.lib.control.component;

import java.util.List;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.api.type.InterfaceType;

import aokell.lib.control.Controller;
import aokell.lib.type.InterfaceTypeImpl;

/**
 * Interface for component implementations.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public interface ComponentItf extends Component {

    final public static String NAME = "component";
    
    final public static InterfaceType TYPE =
        new InterfaceTypeImpl(
                NAME,
                ComponentItf.class.getName(),
                false, false, false);

    /**
     * Return the list of controllers implemented by the component.
     */
    public Controller[] getFcControllers();
    
    /**
     * Declare a new controller implemented by the component.
     */
    public void addFcController( InterfaceType it, Controller ctrl );

    /**
     * Set the content associated to this component.
     */
    public void setContentPart( Object contentPart );

    /**
     * Return the list of client interfaces implemented by this component.
     */
    public List getFcSingletonClientInterfaces();

    /**
     * Return the instance implementing the given client interface name for this
     * component. This method is a clone for getFcInterface(String).
     */
    public Object getFcClientInterface(String interfaceName)
            throws NoSuchInterfaceException;

    /**
     * @return  the list of singleton Interface instances implemented by this
     *          component
     */
    public List getFcSingletonInterfaces();

    /**
     * @return  the list of singleton business Interface instances implemented
     *          by this component
     */
    public List getFcSingletonBusinessInterfaces();

    /**
     * @return  the list of singleton control Interface instances implemented
     *          by this component
     */
    public List getFcSingletonControlInterfaces();

    /**
     * @return  the list of collection Interface instances implemented by this
     *          component
     */
    public List getFcCollectionInterfaces();

    public void setType(Type type);

    public void setControllerDesc(Object controllerDesc);

    public void setContentDesc(Object contentDesc);    
}
