/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package aokell.lib.control.factory;

import java.util.Map;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.factory.Factory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.InterfaceType;

import aokell.lib.type.InterfaceTypeImpl;

/**
 * This interface extends the Fractal Factory interface with a method which
 * provides hints for instantiating a new component.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public interface FactoryItf extends Factory {

    final public static String NAME = "factory";
    
    final public static InterfaceType TYPE =
        new InterfaceTypeImpl(
                NAME,
                FactoryItf.class.getName(),
                false, false, false);

    /**
     * Recursively create a new component and store the association between old
     * instances and newly created ones in the given map.
     * 
     * @param component   the component to clone
     * @param newContent  the map containing associations between old and new
     *                    components
     * @return  the newly created component
     */
    public Component newFcInstance( Component component, Map newContent )
    throws InstantiationException;
    
}