/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 */

package aokell.lib.control.factory;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.api.factory.InstantiationException;

import aokell.lib.control.CloneCtrlException;
import aokell.lib.control.Controller;
import aokell.lib.control.component.ComponentDescItf;
import aokell.lib.control.component.ComponentItf;
import aokell.lib.factory.GenericFactoryImpl;
import aokell.lib.util.FeatureHelper;
import aokell.lib.util.FractalHelper;

/**
 * Implementation of the factory interface for primitive components.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public class PrimitiveFactoryImpl extends AbstractFactoryImpl {

    public PrimitiveFactoryImpl() {
        super();
    }
    

    // --------------------------------------------------------------
    // Factory interface
    // --------------------------------------------------------------
    
    public Component newFcInstance() throws InstantiationException {
        
        Component compctrl = FeatureHelper.getFcImplementation(this.compctrl);

        // Templates are created with something like:
        // Component cTmpl = cf.newFcInstance(
        //      cType, "primitiveTemplate",
        //      new Object[] { "primitive", "ClientImpl" });
        
        // Instantiate the primitive
        Map newContent = new HashMap();
        Component newComponent = newFcInstance(compctrl,newContent);
        
        // Initialize the newly created component by copying the state of
        // the controllers owned by the template
        Controller[] ctrls = ((ComponentItf)compctrl).getFcControllers();
        for (int i = 0; i < ctrls.length; i++) {
            Controller ctrl = ctrls[i];
            
            try {
                ctrl.cloneFcCtrl(newComponent,newContent);
            }
            catch( CloneCtrlException cce ) {
                cce.printStackTrace();
                throw new InstantiationException(cce.getMessage());
            }                                
        }
        
        // Copy the state of the attribute controller
        try {
            copyAttributeControllerState(compctrl,newComponent,newContent);
        }
        catch( CloneCtrlException cce ) {
            cce.printStackTrace();
            throw new InstantiationException(cce.getMessage());
        }                                
        
        return newComponent;
    }
    
    
    // --------------------------------------------------------------
    // FactoryItf interface
    // --------------------------------------------------------------    
    
    /**
     * Recursively create a new component and store the association between old
     * instances and newly created ones in the given map.
     * 
     * @param component   the component to clone
     * @param newContent  the map containing associations between old and new
     *                    components
     * @return  the newly created component
     */
    public Component newFcInstance(Component component, Map newContent)
    throws InstantiationException {
        
        // Templates are created with something like:
        // Component cTmpl = cf.newFcInstance(
        //      cType, "primitiveTemplate",
        //      new Object[] { "primitive", "ClientImpl" });
        
        ComponentDescItf cdi =
            (ComponentDescItf) FractalHelper.getFcInterface(component,"/desc");
        Type type = component.getFcType();
        Object controllerDesc = ((Object[])cdi.getContentDesc())[0];
        Object contentDesc = ((Object[])cdi.getContentDesc())[1];
        
        GenericFactoryImpl gf = GenericFactoryImpl.get();
        
        // Instantiate the current component
        Component newComponent =
            gf.newFcInstance(type,controllerDesc,contentDesc);
        newContent.put( component, newComponent );
        
        return newComponent;
    }
}