/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 * Contributors: Romain Rouvoy
 * -----------------------------------------------------------------------------
 * $Id: CompositeLifeCycleControllerImpl.java,v 1.4 2005/12/19 13:21:20 seinturier Exp $
 * -----------------------------------------------------------------------------
 */
package aokell.lib.control.lifecycle;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;

import aokell.lib.util.FractalHelper;


/**
 * Default implementation of the life cyle controller for composite components.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 */
public class CompositeLifeCycleControllerImpl
    extends AbstractLifeCycleControllerImpl
    implements LifeCycleControllerItf {  // Needed for fractalrmi
    
    // --------------------------------------------------------------
    // LifeCycleController interface methods unimplemented in
    // AbstractLifeCycleControllerImpl
    // --------------------------------------------------------------
    
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    public boolean setFcStarted() throws IllegalLifeCycleException {
        if (!super.setFcStarted())
            return false;
        setFcState(true);
        return true;
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
     */
    public boolean setFcStopped() throws IllegalLifeCycleException {
        if (!super.setFcStopped())
            return false;
        setFcState(false);
        return true;
    }
    
    private void setFcState(final boolean start) throws IllegalLifeCycleException {
        ContentController cc = FractalHelper.getContentController(compctrl);
        Component[] content = cc.getFcSubComponents();

        for (int i = 0; i < content.length; i++) {
            try { // Recursively start or stop all sub-components
                LifeCycleController lc = (LifeCycleController) content[i]
                        .getFcInterface("lifecycle-controller");
                if (start) {
                    if (lc instanceof LifeCycleControllerItf)
                        ((LifeCycleControllerItf) lc).setFcStarted();
                    else
                        lc.startFc();
                } else {
                    if (lc instanceof LifeCycleControllerItf)
                        ((LifeCycleControllerItf) lc).setFcStopped();
                    else
                        lc.stopFc();
                }
            } catch (NoSuchInterfaceException nsie) {
                // NoSuchInterfaceException ignored.
                // throw new IllegalLifeCycleException("No such interface:
                // "+nsie.getMessage() );
            }
        }
    }
}
