/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 * Contributors: Romain Rouvoy
 * -----------------------------------------------------------------------------
 * $Id: LifeCycleControllerItf.java,v 1.5 2005/11/18 13:51:11 rouvoy Exp $
 * -----------------------------------------------------------------------------
 */

package aokell.lib.control.lifecycle;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.type.InterfaceType;

import aokell.lib.type.InterfaceTypeImpl;

/**
 * This interface extends the Fractal LifeCycleController interface with a
 * method to test whether the component is started or not.
 *
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 */
public interface LifeCycleControllerItf extends LifeCycleController {

    /** <code>NAME</code> of the lifecycle controller. */
    final public static String NAME = "lifecycle-controller";
    
    /** <code>TYPE</code> of the lifecycle controller. */
    final public static InterfaceType TYPE =
        new InterfaceTypeImpl(
                NAME,
                LifeCycleControllerItf.class.getName(),
                false, false, false );

    /**
     * @return true if the component is started
     */
    public boolean isFcStarted();
    
    /**
     * @return true if the component is in the STOPPED state before the call.
     * @throws IllegalLifeCycleException
     */
    public boolean setFcStarted() throws IllegalLifeCycleException ;

    /**
     * @return true if the component is in the STARTED state before the call.
     * @throws IllegalLifeCycleException
     */
    public boolean setFcStopped() throws IllegalLifeCycleException ;
}
