/***
 * AOKell
 * Copyright (C) 2005 INRIA, France Telecom
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Lionel.Seinturier@lifl.fr
 *
 * Author: Lionel Seinturier
 * -----------------------------------------------------------------------------
 * $Id: LifeCycleControllerSetterGetterItf.java,v 1.3 2005/11/18 13:51:11 rouvoy Exp $
 * -----------------------------------------------------------------------------
 */

package aokell.lib.control.lifecycle;

/**
 * Interface for managing the reference towards the lifecycle controller.
 * 
 * This interface allows optimizing the access to the lifecycle controller from
 * a component. Instead of getting the membrane, and then the lifecycle
 * controller interface (which is bound with an export binding to the primitive
 * component which implements the lifecycle controller), this interface provides
 * a direct access to the lifecycle controller.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@lifl.fr>
 */
public interface LifeCycleControllerSetterGetterItf {
    /**
     * Provides the reference of the lifecycle controller used.
     * @return the lifecycle controller used by the component.
     */
    public LifeCycleControllerItf getFcLifeCycleController();
    
    /**
     * Defines the lifecycle controller to use.
     * @param _lc the reference of the lifecycle controller to use.
     */
    public void setFcLifeCycleController( LifeCycleControllerItf _lc );
}
